import random

"""
TempSensor simulates a temperature sensor. :)
The constructor parameters are:
    previous: the initial temperature value
    step: the step to increase or decrease the temperature value
Both parameters are optional.
"""
class TempSensor:

    def __init__(self, previous = 30.0, step = 0.5):
        self.previous = previous
        self.step = step

    def get_temp(self):
        if random.randint(0,1) == 1:
            current = self.previous + self.step
        else:
            current = self.previous - self.step

        self.previous = current
        return current



import random


class PHSensor:
    """
    PHSensor simulates a pH sensor for either water or soil.
    The constructor parameters are:
        type: 'water' or 'soil' to specify the type of pH sensor.
        previous: the initial pH value
        step: the step to increase or decrease the pH value
    Both 'previous' and 'step' parameters are optional.
    """

    WATER_PH_RANGE = (6.0, 8.0)  # Typical for clean freshwater
    SOIL_PH_RANGE = (4.0, 9.0)  # Common range for various soils

    def __init__(self, sensor_type='water', previous=7.0, step=0.1):
        if sensor_type not in ['water', 'soil']:
            raise ValueError("Sensor type must be either 'water' or 'soil'")

        self._type = sensor_type
        self._previous = previous
        self._step = step

    def read_ph(self):
        """
        Simulates reading from the pH sensor by either increasing or decreasing
        the previous pH level based on a random step value.
        """
        step_variation = random.uniform(-self._step, self._step)
        current = self._previous + step_variation

        # Ensure pH level remains within the defined range based on sensor type
        if self._type == 'water':
            current = max(self.WATER_PH_RANGE[0], min(self.WATER_PH_RANGE[1], current))
        else:
            current = max(self.SOIL_PH_RANGE[0], min(self.SOIL_PH_RANGE[1], current))

        self._previous = current
        return current



import random


class UVSensor:
    """
    UVSensor simulates an ultraviolet (UV) sensor.
    The constructor parameters are:
        previous: the initial UV index value
        step: the step to increase or decrease the UV index value
    Both parameters are optional.
    """

    MIN_UV_INDEX = 0.0  # Lowest UV index value
    MAX_UV_INDEX = 11.0  # Extreme UV index value

    def __init__(self, previous=5.0, step=0.5):
        self._previous = previous
        self._step = step

    def read_uv_index(self):
        """
        Simulates reading from the UV sensor by either increasing or decreasing
        the previous UV index based on a random step value.
        """
        step_variation = random.uniform(-self._step, self._step)
        current = self._previous + step_variation

        # Ensure UV index remains within the defined range
        current = max(self.MIN_UV_INDEX, min(self.MAX_UV_INDEX, current))

        self._previous = current
        return current



import random
from enum import Enum


class SoilState(Enum):
    AIR = 1
    DRY = 2
    MOIST = 3
    WATER = 4


def get_soil_moisture_level():
    #return a tumple consisting of an integer value and an enum value that describe
    # the moisture in the soil
    current = random.randint(1,1000)
    if current >= 1000:
        state = SoilState.AIR
    elif current < 1000 and current >= 600:
        state = SoilState.DRY
    elif current < 600 and current >= 370:
        state = SoilState.MOIST
    else:
        state = SoilState.WATER

    return current, state



from distutils.core import setup
from pathlib import Path

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
  name = 'sim_sensors',
  packages = ['sim_sensors'],
  version = '0.3',
  license='MIT',
  description = 'A collection of classes that simulate IoT sensors',
  long_description=long_description,
  author = 'Ivan Šarić / Path Variable',
  author_email = 'ivan@path-variable.com',
  url = 'https://www.path-variable.com',
  keywords = ['simulated', 'sensors', 'iot'],
  install_requires=[],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)


import random


class LightSensor:
    """
    LightSensor simulates an ambient light sensor.
    The constructor parameters are:
        previous: the initial light value in lux
        step: the step to increase or decrease the light value
    Both parameters are optional.
    """

    MIN_LUX = 100.0  # Dimly lit room
    MAX_LUX = 1000.0  # Very brightly lit room

    def __init__(self, previous=500.0, step=50.0):
        self._previous = previous
        self._step = step

    def read_illuminance(self):
        """
        Simulates reading from the light sensor by either increasing or decreasing
        the previous light level based on a random step value.
        """
        step_variation = random.uniform(-self._step, self._step)
        current = self._previous + step_variation

        # Ensure light level remains within the defined range
        current = max(self.MIN_LUX, min(self.MAX_LUX, current))

        self._previous = current
        return current



import random


class GasSensor:
    """
    GasSensor simulates various gas sensors including CO2, methane, CO, and air quality.
    The constructor parameters are:
        gas_type: Specifies the type of gas ('CO2', 'methane', 'CO', 'AQI').
        previous: the initial gas concentration
        step: the step to increase or decrease the gas concentration
    Both 'previous' and 'step' parameters are optional.
    """

    GAS_RANGES = {
        'CO2': (400.0, 5000.0),  # ppm
        'methane': (0.5, 5.0),  # ppm
        'CO': (0.0, 50.0),  # ppm
        'AQI': (0, 500)  # Index
    }

    def __init__(self, gas_type='CO2', previous=None, step=10.0):
        if gas_type not in self.GAS_RANGES:
            raise ValueError("Invalid gas type. Choose from 'CO2', 'methane', 'CO', or 'AQI'.")

        self._type = gas_type
        self._previous = previous if previous else self.GAS_RANGES[gas_type][0]
        self._step = step

    def read_concentration(self):
        """
        Simulates reading from the gas sensor by either increasing or decreasing
        the previous gas concentration based on a random step value.
        """
        step_variation = random.uniform(-self._step, self._step)
        current = self._previous + step_variation

        # Ensure concentration remains within the defined range for the gas type
        current = max(self.GAS_RANGES[self._type][0], min(self.GAS_RANGES[self._type][1], current))

        self._previous = current
        return current



import random


class PressureSensor:
    """
    PressureSensor simulates a barometric pressure sensor.
    The constructor parameters are:
        previous: the initial pressure value in hPa
        step: the step to increase or decrease the pressure value
    Both parameters are optional.
    """

    MIN_PRESSURE = 950.0  # hPa, might be seen during a strong storm
    MAX_PRESSURE = 1050.0  # hPa, high pressure during cold, clear conditions

    def __init__(self, previous=1013.25, step=1.0):
        self._previous = previous
        self._step = step

    def read_pressure(self):
        """
        Simulates reading from the pressure sensor by either increasing or decreasing
        the previous pressure level based on a random step value.
        """
        step_variation = random.uniform(-self._step, self._step)
        current = self._previous + step_variation

        # Ensure pressure remains within the defined range
        current = max(self.MIN_PRESSURE, min(self.MAX_PRESSURE, current))

        self._previous = current
        return current
